/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Copyright (C) 2002 Motorola Semiconductors HK Ltd
 *
 */

/******************************************************************************
 *
 * Copyright (C) 2001, Motorola All Rights Reserved
 *
 * File Name:   mmcsd_cmd.c
 *
 * Progammers:  Yiwei Zhao
 *
 * Date of Creations:   30  DEC,2001
 *
 * Synopsis:
 *
 * Modification History:
 * 30 DEC, 2001
 *
 *****************************************************************************/

#include <system_def.h>
#include <mx1_reg.h>
#include "mmcsd_def.h"
#include "mmcsd_main.h"

/* Functions List:
 *
 */

/******************************************************************************
 * Function Name: _MMCSD_GoIdleState
 *
 * Input:               None.
 *
 * Value Returned:
 *
 * 	MMCSD_STATUS
 *
 * Description:
 *	Resets all cards to idle state
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/

MMCSD_STATUS _MMCSD_GoIdleState(MMCSD_Dev *device)
{
	MMCSD_STATUS ret;
	ret=_MMCSD_NoDataCommand(device, MMCSD_CMD0,0,MMCSDB_INIT);
	return ret;
}

/******************************************************************************
 * Function Name: _MMCSD_SendOpCond
 *
 * Input:
 * 		voltage
 *
 * Value Returned:
 *
 * Description:
 *	Ask all cards in idle state to send their operation conditions register
 *	contern in response on the CMD line.
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/

MMCSD_STATUS _MMCSD_SendOpCond(MMCSD_Dev *device, uint32_t voltage)
{
	MMCSD_Responses rp;
	MMCSD_STATUS ret;

	// Try first for a SD card
	ret = _MMCSD_NoDataCommand(device, MMCSD_CMD55, 0, MMCSDB_R1);

	if (ret == 0)
	{
		// Read response of CMD55
		_MMCSD_GetResponses(device, &rp, MMCSDB_R1);

		device->card_type = MMCSD_TYPE_SD;
		ret = _MMCSD_NoDataCommand(device, MMCSD_ACMD41, voltage, MMCSDB_R3);
	}
	else
	{
		device->card_type = MMCSD_TYPE_MMC;

		//voltage = 0xFFFFFF0;
		ret = _MMCSD_NoDataCommand(device, MMCSD_CMD1, voltage, MMCSDB_R3);
	}

	_MMCSD_GetResponses(device, &rp, MMCSDB_R3);

	if (ret)
	{
		TRACE("CMD1 failed with ret = 0x%lx\n",(unsigned long)ret);
		return ret;
	}


	if ((rp.OCR & 0x80000000) == 0)
		return MMCSD_CARD_BUSY;
	else
	{
		//TRACE("OCR is %lx\n",rp.OCR);
		return 0;
	}

}

/******************************************************************************
 * Function Name: _MMCSD_AllSendCID
 *
 * Input:
 * 		uint8_t * pCidBuff
 *
 * Value Returned:
 *
 * 	MMCSD_STATUS
 *
 * Description:
 *
 *	CMD2:  Ask all cards to send their CID numbers on rhe CMD line
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/
/*****************************************************************************

  CID Register:(Bit)    127   119 111  103 95  87   79  71    63  55  47  39  31  15  7   0
  ------------------------------------------------------------------------------------------
  CID Register          |MID  |  OID   |        PNM               |     PSN       |MDT| CRC|
  -------------------------------------------------------------------------------------------
pCidBuff    :(BYTE)   |  0  | 1 |  2 | 4 | 5 | 7 | 8  |  9  | 10| 11| 12|13| 14 | 15 | 16|


 ******************************************************************************/
MMCSD_STATUS _MMCSD_AllSendCID(MMCSD_Dev *device, uint8_t *pCidBuff)
{
	MMCSD_Responses rp;
	MMCSD_STATUS ret;
	int i;

	ret=_MMCSD_NoDataCommand(device, MMCSD_CMD2,0,MMCSDB_R2);

	if(ret)
	{
		FAILED("Command 2 returns with %d\n",ret);
		return ret;
	}

	_MMCSD_GetResponses(device, &rp,MMCSDB_R2);

	for(i=0;i<16;i++)
	{
		pCidBuff[i]=rp.CID[i];
	}
	return 0;
}

/******************************************************************************
 * Function Name: _MMCSD_SetRelativeAddr
 *
 * Input:
 * 		addr		relative address
 *
 * Value Returned:
 *
 * 	MMCSD_STATUS
 *
 * Description:
 *
 * 	Assigns relative address to the card
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *
 * Qeustion:
 *
 * 	assigned or generated by itself?
 *
 *****************************************************************************/
MMCSD_STATUS _MMCSD_SetRelativeAddr(MMCSD_Dev *device, uint32_t *addr)
{
	MMCSD_Responses rp;
	MMCSD_STATUS ret;

	ret = _MMCSD_NoDataCommand(device, MMCSD_CMD3,
			(device->card_type == MMCSD_TYPE_MMC) ? *addr << 16 : 0,
			MMCSDB_R1);

	if (ret)
		return ret;

	_MMCSD_GetResponses(device, &rp, MMCSDB_R1);

	if (device->card_type == MMCSD_TYPE_SD)
	{
		*addr = (rp.status >> 16) & 0xFFFF;
		return 0;
	}
	else
		return rp.status & MMCSD_STATUS_NOERROR;
}

/******************************************************************************
 * Function Name: _MMCSD_SelectDeselectCard
 *
 * Input:
 *
 * 		addr:		relative address
 *
 * Value Returned:
 *
 * 	MMCSD_STATUS
 *
 * Description:
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/
MMCSD_STATUS _MMCSD_SelectDeselectCard(MMCSD_Dev *device, uint32_t addr)
{
	MMCSD_Responses rp;
	MMCSD_STATUS ret;

	if(addr!=0)
		ret=_MMCSD_NoDataCommand(device, MMCSD_CMD7,addr<<16,0x40|MMCSDB_R1);
	else
		ret=_MMCSD_NoDataCommand(device, MMCSD_CMD7,0,0x40);

	if(ret)
		return ret;

	_MMCSD_GetResponses(device, &rp,MMCSDB_R1);


	return rp.status&MMCSD_STATUS_NOERROR;

}

/******************************************************************************
 * Function Name: _MMCSD_SendCSD
 *
 * Input:
 * 		addr:	relative address.
 *
 * Value Returned:
 *
 * 	MMCSD_STATUS
 *
 * Description:
 *		CMD9:       Addressed card sends its card-specific data(CSD)
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/
/*****************************************************************************

  CSD Register
  -----------------------------------------------------------------------------
  Name                        | Width| CSD_Slice|    pCsdbuff (BYTE)
  -----------------------------------------------------------------------------
  CSD struction				   2     127:126
  System spec. version           4     125:122           0
  reserved                       2     121:120
  -----------------------------------------------------------------------------
  data read access time 1        8     119:112           1
  -----------------------------------------------------------------------------
  data read access time 2        8     111:104           2
  -----------------------------------------------------------------------------
  Max data transfer rate         8     103:96            3
  -----------------------------------------------------------------------------
  card command classes           12     95:84            4
  max read data block length     4      83:80            5
  ------------------------------------------------------------------------------
  partial blocks for read length  1     79:79
  wirte block misalignment        1     78:78
  read block misalignment         1     77:77
  DSR implemented                 1     76:76
  reserved                        2     75:74
  device size                    12     73:62            6-7
  max read current                3     61:69
  max read current                3     58-56            8
  ------------------------------------------------------------------------------
  max write current               3     55-53
  max write current               3     52-50
  device size multiplier          3     49-47            9
  erase sector size               5     46-42
  erase group size                5     41-37            10
  write protect group size        5     36-32            11
  ------------------------------------------------------------------------------
  write protect group enable      1     31:31
  manufactuter default ECC        2     30:29
  write speed factor              3     28:26
  max. write data block length    4     25:22             12
  partial blocks for write allowed 1    21:21
  reserved                         5    20:16             13
  ------------------------------------------------------------------------------
  File format group               1     15:15
  copy flag                       1     14:14
  permanent write protection      1     13:13
  temporary write protection      1     12:12
  file format                     2     11:10
  ECC code                        2      9:8              14
  --------------------------------------------------------------------------------
  CRC                             7      7:1
  Not used   always '1'           1      0:0              15
  -------------------------------------------------------------------------------

 ******************************************************************************/
MMCSD_STATUS _MMCSD_SendCSD(MMCSD_Dev *device, uint32_t addr,uint8_t* pBuff)
{
	MMCSD_Responses rp;
	MMCSD_STATUS ret;
	int i;

	ret=_MMCSD_NoDataCommand(device, MMCSD_CMD9,addr<<16,MMCSDB_R2);

	if(ret)
		return ret;

	_MMCSD_GetResponses(device, &rp,MMCSDB_R2);

	for(i=0;i<16;i++)
	{
		pBuff[i]=rp.CSD[i];
		TRACE(" %02x", pBuff[i]);
	}

	return 0;
}

/******************************************************************************
 * Function Name: _MMCSD_SendCID
 *
 * Input:
 *		addr:		relative address.
 *
 * Value Returned:
 *
 * 	MMCSD_STATUS
 *
 * Description:
 *		CMD10: Addressed card sends its card-specific data(CSD)
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/
MMCSD_STATUS _MMCSD_SendCID(MMCSD_Dev *device, uint32_t addr,uint8_t * pBuff)
{
	MMCSD_Responses rp;
	MMCSD_STATUS ret;
	int i;

	ret=_MMCSD_NoDataCommand(device, MMCSD_CMD10,addr<<16,MMCSDB_R2);

	if(ret)
		return ret;

	_MMCSD_GetResponses(device, &rp,MMCSDB_R2);
	for(i=0;i<16;i++)
	{
		pBuff[i]=rp.CID[i];
	}

	return 0;
}

/******************************************************************************
 * Function Name: _MMCSD_SendStatus
 *
 * Input:               .
 *
 * Value Returned:
 *
 * 	MMCSD_STATUS
 *
 * Description: CMD13	addressed card sends its status register
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/
MMCSD_STATUS  _MMCSD_SendStatus(MMCSD_Dev *device, uint32_t addr,uint32_t *pStatus)
{
	MMCSD_Responses rp;
	MMCSD_STATUS ret;

	ret=_MMCSD_NoDataCommand(device, MMCSD_CMD13,addr<<16,MMCSDB_R1);
	if(ret)
	{
		//FAILED("CMD13 Failed with ret = 0x%x\n",ret);
		*pStatus = ret;
		return ret;
	}

	_MMCSD_GetResponses(device, &rp,MMCSDB_R1);

	*pStatus=rp.status;

	return ((rp.status)&MMCSD_STATUS_NOERROR);

}

/******************************************************************************
 * Function Name: _MMCSD_SetBlockLen
 *
 * Input:               None.
 *
 * Value Returned:
 *
 * 	MMCSD_STATUS
 *
 * Description:
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/
MMCSD_STATUS _MMCSD_SetBlockLen(MMCSD_Dev *device, uint32_t size, uint32_t num)
{
	MMCSD_Responses rp;
	MMCSD_STATUS ret;

	_MMCSD_SetBlockLenReg(device, size);
	_MMCSD_SetNumberOfBlockReg(device, num);

	ret=_MMCSD_NoDataCommand(device, MMCSD_CMD16,size,MMCSDB_R1);

	if(ret)
	{
		FAILED("setBlockLen ret= %x\n",ret);
		return ret;
	}
	_MMCSD_GetResponses(device, &rp,MMCSDB_R1);

	return rp.status&MMCSD_STATUS_NOERROR;
}

/******************************************************************************
 * Function Name: _MMCSD_TagSectorStart
 *
 * Input:            uint32_t  addr       address
 *
 * Value Returned:
 *
 * 	MMCSD_STATUS
 *
 * Description:
 *	CMD33  Sets the address of the first sector of the erase group
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/
MMCSD_STATUS _MMCSD_TagSectorStart(MMCSD_Dev *device, uint32_t addr)
{
	MMCSD_Responses rp;
	MMCSD_STATUS ret;

	ret=_MMCSD_NoDataCommand(device, MMCSD_CMD32,addr,MMCSDB_R1);

	if(ret)
		return ret;

	_MMCSD_GetResponses(device,&rp,MMCSDB_R1);

	return rp.status&MMCSD_STATUS_NOERROR;

}
/******************************************************************************
 * Function Name: _MMCSD_TagSectorEnd
 *
 * Input:            uint32_t  addr       address
 *
 * Value Returned:
 *
 * 	MMCSD_STATUS
 *
 * Description:
 *	CMD33  Sets the address of the last sector of the erase group
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/

MMCSD_STATUS _MMCSD_TagSectorEnd(MMCSD_Dev *device, uint32_t addr)
{
	MMCSD_Responses rp;
	MMCSD_STATUS ret;

	ret=_MMCSD_NoDataCommand(device, MMCSD_CMD33,addr,MMCSDB_R1);


	if(ret)
		return ret;

	_MMCSD_GetResponses(device,&rp,MMCSDB_R1);

	return rp.status&MMCSD_STATUS_NOERROR;

}

/******************************************************************************
 * Function Name: _MMCSD_Erase
 *
 * Input:
 *
 * Value Returned:
 *
 * 	MMCSD_STATUS
 *
 * Description:
 *	Erase all previous selected blocks.
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/
MMCSD_STATUS _MMCSD_Erase(MMCSD_Dev *device)
{
	MMCSD_Responses rp;
	MMCSD_STATUS ret;

	ret=_MMCSD_NoDataCommand(device, MMCSD_CMD38,0,MMCSDB_R1);

	if(ret)
		return ret;

	_MMCSD_GetResponses(device, &rp,MMCSDB_R1);


	return rp.status&MMCSD_STATUS_NOERROR;

}

/******************************************************************************
 * Function Name: _MMCSD_NoDataCommand
 *
 * Input:
 * 	cmd 		Command Number
 *	arg		Command argument
 *	ctrl		Command control value inculding response type
 *
 * Value Returned:
 *
 * 	MMCSD_STATUS
 *
 * Description:
 *	Send commend to MMC card and wait for response
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/

MMCSD_STATUS _MMCSD_NoDataCommand(MMCSD_Dev *device, uint8_t cmd, uint32_t arg,uint32_t ctrl)
{
	MMCSD_STATUS ret;
	uint32_t	int_mask;
	int     timeout = 1000;

	switch(cmd)
	{
		case MMCSD_CMD16:
			timeout =500 ;
			break;

		case MMCSD_CMD6:
			timeout = 30;
			break;

		case MMCSD_CMD25:
			timeout = 500;
			break;
		case MMCSD_CMD38:
			timeout = 1000*60*5;
			break;
		default :
			timeout = 1000;

	}
	device->cur_cmd = cmd;
	_MMCSD_Mask_interrupt(device, 0x7b,&int_mask);

	_MMCSD_StopClk(device);

	device->pending_irq = 0;

	_MMCSD_SetCmd(device, cmd,arg,ctrl);
	_MMCSD_StartClk(device);
	if((cmd==MMCSD_CMD13)||(cmd==MMCSD_CMD55))
	{
		ret=_MMCSD_WaitUntil(device, MMCSD_CMD_DONE);
	}
	else
	{
		ret=_MMCSD_irq_WaitUntil(device, MMCSD_CMD_DONE,cmd,timeout);
	}

	_MMCSD_Restore_Mask(device, int_mask);

	return ret;
}

MMCSD_STATUS _MMCSD_irq_WaitUntil(MMCSD_Dev *device, uint32_t EndCondition,uint8_t cmd,int timeout)
{
	uint32_t	   status,timeleft;

	if(device==NULL)
	{						//polling
		//TRACE("device is NULL\n");
		do
		{
			status = mmcsd_reg_rd(device, MX1_MMCSD_STATUS_o);
		}
		while ((status&0x2002)==0);

		if(status&MMCSD_CMD_TIMEOUT)
		{
			FAILED("CMD Result: 0x%04x\n",mmcsd_reg_rd(device, MX1_MMCSD_STATUS_o));
		}

		return (status&MMCSD_CMD_TIMEOUT);
	}

	status = mmcsd_reg_rd(device, MX1_MMCSD_STATUS_o);		// Sometimes, the interrupt has happened.

	if((status&(MMCSD_STATUS_ERR|MMCSD_END_CMD_RESP))==0)
	{
		timeleft = mmcsd_device_select_wait(device,timeout);
		if(!timeleft)
		{
			status = mmcsd_reg_rd(device, MX1_MMCSD_STATUS_o);		// Sometimes, the interrupt has happened.

			if((status&(MMCSD_STATUS_ERR|MMCSD_END_CMD_RESP))==0)
			{
				FAILED("Time Out, MMCSD_STATUS = 0x%lx, cmd%d\n",(unsigned long)status,cmd);
				return MMCSD_ERROR;
			}
			TRACE("Time Out!,cmd%d\n",cmd);
		}
		else
		{
			status = device->result;
		}
	}
	if(status&MMCSD_STATUS_ERR)
	{
		//FAILED("error, status is 0x%0lX,(cmd%d)\n",status,cmd);
		return(status&MMCSD_STATUS_ERR);
	}

	if(EndCondition == MMCSD_CMD_DONE)
	{
		if((status&MMCSD_END_CMD_RESP)==0)
		{
			FAILED("error, status is 0x%0lX,(cmd%d)\n",(unsigned long)status,cmd);
			if(status != mmcsd_reg_rd(device, MX1_MMCSD_STATUS_o))
				FAILED("but actual status is 0x%0lX\n",(unsigned long)mmcsd_reg_rd(device, MX1_MMCSD_STATUS_o));
			return(MMCSD_ERROR);
		}
	}

	return (MMCSD_OK);

}

void _MMCSD_Mask_interrupt(const MMCSD_Dev *device, uint32_t mask, uint32_t *orig)
{
	*orig = mmcsd_reg_rd(device, MX1_MMCSD_INT_MASK_o);
	mmcsd_reg_wr(device, MX1_MMCSD_INT_MASK_o, mask);
	TRACE("Int_mask is 0x%0lX\n",(unsigned long)mmcsd_reg_rd(device, MX1_MMCSD_INT_MASK_o));
}

void _MMCSD_Restore_Mask(const MMCSD_Dev *device, uint32_t orig)
{
	mmcsd_reg_wr(device, MX1_MMCSD_INT_MASK_o, orig);
	//	TRACE("MMCSD_INT_MASK = 0x%x\n",mmcsd_reg_rd(device, MX1_MMCSD_INT_MASK_o));
}

/******************************************************************************
 * Function Name: _MMCSD_WaitUntil
 *
 * Input:
 *
 * Value Returned:
 *
 * 	MMCSD_STATUS
 *
 * Description:
 *	Wait until Command done
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/
MMCSD_STATUS _MMCSD_WaitUntil(MMCSD_Dev *device, uint32_t EndCondition)
{
	//waiting for the End Cmd Response
	volatile uint32_t status;
	uint32_t counter = 0xFFFFFFFF;

	do{
		status = mmcsd_reg_rd(device, MX1_MMCSD_STATUS_o);

		counter--;
		if(counter==0)
		{
			TRACE("Time Out.\n");
			return MMCSD_ERROR;
		}
		//TRACE("staus is %lx\n",status);
	}
	while ((status&(MMCSD_END_CMD_RESP|MMCSD_STATUS_ERR))==0);


	/*FIXME: MMCSD_CMD_TIMEOUT has not been checked on MX2, is that intentional or by mistake */
	/*status&(MMCSD_READ_TIMEOUT|MMCSD_WRITE_CRC)*/
#ifdef CONFIG_ARCH_MX2ADS
	if(status&(MMCSD_READ_TIMEOUT|MMCSD_WRITE_CRC) != 0)
		FAILED(" CMD Result: 0x%04x\n",mmcsd_reg_rd(device, MX1_MMCSD_STATUS_o));

	return (status&(MMCSD_READ_TIMEOUT|MMCSD_WRITE_CRC));
#endif
#ifdef CONFIG_ARCH_MX1ADS
	if(status&MMCSD_STATUS_ERR)
		FAILED(" CMD Result: 0x%04x\n",mmcsd_reg_rd(device, MX1_MMCSD_STATUS_o));
	return (status&MMCSD_STATUS_ERR);
#endif

}

/******************************************************************************
 * Function Name: _MMCSD_SetWriteProt
 *
 * Input:
 *
 * Value Returned:
 *
 * 	MMCSD_STATUS
 *
 * Description:
 * 	this routine will set the write protection bit for the addressed
 * 	group.
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/
MMCSD_STATUS _MMCSD_SetWriteProt(MMCSD_Dev *device, uint32_t addr)
{
	MMCSD_Responses rp;
	MMCSD_STATUS ret;

	ret=_MMCSD_NoDataCommand(device, MMCSD_CMD28,addr,MMCSDB_R1);

	if(ret)
		return ret;

	_MMCSD_GetResponses(device, &rp,MMCSDB_R1);


	return rp.status&MMCSD_STATUS_NOERROR;

}
/******************************************************************************
 * Function Name: _MMCSD_ClearWriteProt
 *
 * Input:
 *
 * Value Returned:
 *
 * 	MMCSD_STATUS
 *
 * Description:
 *	This routine will clear the write protection bit for the addressed
 *	group.
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/
MMCSD_STATUS _MMCSD_ClearWriteProt(MMCSD_Dev *device, uint32_t addr)
{
	MMCSD_Responses rp;
	MMCSD_STATUS ret;

	ret=_MMCSD_NoDataCommand(device,MMCSD_CMD29,addr,MMCSDB_R1);

	if(ret)
		return ret;

	_MMCSD_GetResponses(device,&rp,MMCSDB_R1);


	return rp.status&MMCSD_STATUS_NOERROR;

}

MMCSD_STATUS _MMCSD_SetBusWidth(MMCSD_Dev *device, uint32_t rca, uint32_t width)
{
	MMCSD_Responses rp;
	MMCSD_STATUS ret;
	uint32_t	para;

	//TRACE("Set Bus Width =%d\n",width);
	ret=_MMCSD_NoDataCommand(device, MMCSD_CMD55,rca<<16,MMCSDB_R1);
	if(ret)
	{
		TRACE("CMD55 failed with ret = 0x%lx\n",(unsigned long)ret);
		return ret;
	}
	_MMCSD_GetResponses(device, &rp,MMCSDB_R1);

	//	 para = (rca<<16)|width;
	para = width;
	ret=_MMCSD_NoDataCommand(device, MMCSD_ACMD6,para,MMCSDB_R1);
	if(ret)
	{
		TRACE("ACMD6 failed with ret = 0x%lx\n",(unsigned long)ret);
		return ret; //meet some error
	}

	_MMCSD_GetResponses(device, &rp, MMCSDB_R1);

	return MMCSD_OK;
}
/*
   MMCSD_STATUS _MMCSD_SendSCR(MMCSD_Dev *device, uint32_t rca,MMCSD_Responses *rp)
   {
   MMCSD_STATUS ret;

   ret=_MMCSD_NoDataCommand(MMCSD_CMD55,rca<<16,MMCSDB_R1);
   if(ret)
   {
   FAILED("CMD55 failed with ret = 0x%lx\n",ret);
   return ret;
   }
   _MMCSD_GetResponses(rp,MMCSDB_R1);

   ret=_MMCSD_NoDataCommand(MMCSD_ACMD51,0,MMCSDB_R1);
   if(ret)
   {
   FAILED("ACMD6 failed with ret = 0x%lx\n",ret);
   return ret; //meet some error
   }

   _MMCSD_GetResponses(rp,MMCSDB_R1);

   return MMCSD_OK;
   }
 */
