#include <system_def.h>
#include <rtems.h>
#include <rtems/io.h>
#include <rtems/libio.h>
#include <rtems/error.h>
#include <rtems/blkdev.h>
#include <rtems/diskdevs.h>
#include <rtems/fsmount.h>
#include <rtems/ide_part_table.h>
#include <rtems/dosfs.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <mx1_reg.h>
#include "mmcsd_def.h"
#include "mmcsd_main.h"

#define MMCSD_DEVICE_NAME "/dev/mmcsd"


int mmcsd_device_lock_init(MMCSD_Dev *device)
{
	rtems_status_code status;

	status=rtems_semaphore_create(rtems_build_name('S','D','L','0'+device->port),
		1,
		RTEMS_PRIORITY|RTEMS_BINARY_SEMAPHORE|RTEMS_INHERIT_PRIORITY|RTEMS_LOCAL,
		0,
		&device->lock);

	if (status != RTEMS_SUCCESSFUL){
		FAILED(("mmcsd_device_lock_init: cannot create lock %s\n",rtems_status_text(status));
		return status;
	}
	return 0;
}

int mmcsd_device_lock(MMCSD_Dev *device)
{
	rtems_status_code status;

	status=rtems_semaphore_obtain(device->lock,RTEMS_WAIT,10000);

	if (status != RTEMS_SUCCESSFUL){
		FAILED(("mmcsd_device_lock: error %s\n",rtems_status_text(status));
		return -1;
	}
	return 0;
}

int mmcsd_device_unlock(MMCSD_Dev *device)
{
	rtems_status_code status;

	status=rtems_semaphore_release(device->lock);

	if (status != RTEMS_SUCCESSFUL){
		FAILED(("mmcsd_device_unlock: error %s\n",rtems_status_text(status));
		return -1;
	}

	return 0;
}

int mmcsd_device_sem_init(MMCSD_Dev *device)
{
	rtems_status_code status;

	status=rtems_semaphore_create(rtems_build_name('S','D','S','0'+device->port),
		1,
		RTEMS_PRIORITY|RTEMS_BINARY_SEMAPHORE|RTEMS_INHERIT_PRIORITY|RTEMS_LOCAL,
		0,
		&device->sema);

	if (status != RTEMS_SUCCESSFUL){
		FAILED(("mmcsd_device_sem_init: cannot create semaphore %s\n",rtems_status_text(status));
		return status;
	}
	return 0;
}

int mmcsd_device_sem_obtain(MMCSD_Dev *device)
{
	rtems_status_code status;

	status=rtems_semaphore_obtain(device->sema,RTEMS_WAIT,10000);

	if (status != RTEMS_SUCCESSFUL){
		FAILED(("mmcsd_device_sem_obtain: error %s\n",rtems_status_text(status));
		return -1;
	}
	return 0;

}

int mmcsd_device_sem_release(MMCSD_Dev *device)
{
	rtems_status_code status;

	status=rtems_semaphore_release(device->sema);

	if (status != RTEMS_SUCCESSFUL){
		FAILED(("mmcsd_device_sem_release: error %s\n",rtems_status_text(status));
		return -1;
	}

	return 0;
}

int mmcsd_device_select_init(MMCSD_Dev *device)
{
	device->select_wait=MMCSD_INNVALID_TID;
	return 0;
}

long mmcsd_device_select_wait(MMCSD_Dev *device, long timeout)
{
	rtems_interval	ticks=timeout;
	rtems_interval	now;
	rtems_status_code status;
	rtems_event_set   events;
	rtems_id	self_tid;

	if(!ticks)
		ticks = RTEMS_NO_TIMEOUT;

	rtems_task_ident(RTEMS_SELF, 0, &self_tid);
	device->select_wait=self_tid;

	if(device->pending_irq){
		/* The IRQ has already arrived before this function call
		 * clear stalled events and return immediatelly
		 */
		rtems_event_receive (MMCSD_WAIT_EVENT, RTEMS_EVENT_ANY|RTEMS_NO_WAIT, 0, &events);
		status=RTEMS_SUCCESSFUL;
		ticks=0;
		TRACE(("mmcsd_device_select_wait: immediate return\n");
	}else{
		/* Wait for IRQ event
		 */
		rtems_clock_get(RTEMS_CLOCK_GET_TICKS_SINCE_BOOT,&now);
		status=rtems_event_receive (MMCSD_WAIT_EVENT, RTEMS_EVENT_ANY|RTEMS_WAIT, ticks, &events);
		rtems_clock_get(RTEMS_CLOCK_GET_TICKS_SINCE_BOOT,&ticks);
		ticks-=now;
	}
	device->select_wait = MMCSD_INNVALID_TID;
	device->pending_irq = 0;

	if(status==RTEMS_SUCCESSFUL){
		uint32_t status;

		TRACE(("mmcsd_device_select_wait: returned after %d ticks, irq #%d\n",
				ticks,g_mmcsd_irq_count);

		if(!(device->result&MX1_MMCSD_STATUS_END_CMD_RESP_m)){
			/*ERRATUM 4 workaround*/
			rtems_task_wake_after(2);
			status=mmcsd_reg_rd(device, MX1_MMCSD_STATUS_o);
			if(device->result!=status){
				TRACE(("mmcsd_device_select_wait: result 0x%08x != status 0x%08x, irq #%d ERRATUM #4?\n",
					device->result,status,g_mmcsd_irq_count);
				device->result|=status;
			}
		}
		if(timeout-ticks>0)
			return timeout-ticks;
		return 1;
	}

	if(status!=RTEMS_TIMEOUT){
		FAILED(("mmcsd_device_select_wait: error %s\n",rtems_status_text(status));
		return 0;
	}

	if(0){
		uint32_t status=mmcsd_reg_rd(device, MX1_MMCSD_STATUS_o);
		mmcsd_reg_set(device, MX1_MMCSD_INT_MASK_o, 0x40);
		device->result=status;
		FAILED(("mmcsd_device_select_wait: timeout %ld suppressed\n",(long)ticks);
		return 1;
	}

	FAILED(("mmcsd_device_select_wait: timeout %ld\n",(long)ticks);
	return 0;
}

int mmcsd_device_select_wake(MMCSD_Dev *device)
{
	if(device->select_wait!=MMCSD_INNVALID_TID)
		rtems_event_send(device->select_wait, MMCSD_WAIT_EVENT);
	return 0;
}


/* mmcsd_read --
 *     MMC/SD card READ request handler. This primitive copies data from
 *     the card to supplied buffer and invoke the callout function to inform
 *     upper layer that reading is completed.
 *
 * PARAMETERS:
 *     req - pointer to the READ block device request info
 *
 * RETURNS:
 *     ioctl return value
 */
static int
mmcsd_read(MMCSD_Dev *device, blkdev_request *req)
{
	uint32_t    i;
	blkdev_sg_buffer *sg;
	uint32_t    remains;
	uint32_t    bufnum;
	blkdev_bnum sector;
	uint32_t    nr_sector;
	int ret;

	sector = req->start;	/* Start block number */
	remains = req->count;	/* Number of blocks to be exchanged */
	bufnum = req->bufnum;	/* Start block number */
	sg = req->bufs;		/* List of scatter/gather buffers */

	TRACE(("mmcsd_read: start sector %d count %d\n",sector,remains);

	for (i = 0; (remains > 0) && bufnum; i++, sg++, bufnum--)
	{
		int count = sg->length;
		nr_sector = count>>device->blksize_bit;

		if (nr_sector > remains)
        		nr_sector = remains;

		ret=mmcsd_read_device(device, sg->buffer, sector, nr_sector);
		if(ret!=MMCSD_OK){
			req->req_done(req->done_arg, RTEMS_IO_ERROR, EIO);
			return 0;
		}
		remains -= nr_sector;
		sector += nr_sector;
	}
	req->req_done(req->done_arg, RTEMS_SUCCESSFUL, 0);
	return 0;
}

/* mmcsd_write --
 *     MMC/SD card WRITE request handler. This primitive copies data from
 *     the card to supplied buffer and invoke the callout function to inform
 *     upper layer that reading is completed.
 *
 * PARAMETERS:
 *     req - pointer to the READ block device request info
 *
 * RETURNS:
 *     ioctl return value
 */
static int
mmcsd_write(MMCSD_Dev *device, blkdev_request *req)
{
	uint32_t    i;
	blkdev_sg_buffer *sg;
	uint32_t    remains;
	uint32_t    bufnum;
	blkdev_bnum sector;
	uint32_t    nr_sector;
	int ret;

	sector = req->start;	/* Start block number */
	remains = req->count;	/* Number of blocks to be exchanged */
	bufnum = req->bufnum;	/* Start block number */
	sg = req->bufs;		/* List of scatter/gather buffers */

	TRACE(("mmcsd_write: start sector %d count %d\n",sector,remains);

	for (i = 0; (remains > 0) && bufnum; i++, sg++, bufnum--)
	{
		int count = sg->length;
		nr_sector = count>>device->blksize_bit;

		if (nr_sector > remains)
        		nr_sector = remains;

		ret=mmcsd_write_device(device, sg->buffer, sector, nr_sector);
		if(ret!=MMCSD_OK){
			req->req_done(req->done_arg, RTEMS_IO_ERROR, EIO);
			return 0;
		}
		remains -= nr_sector;
		sector += nr_sector;
	}
	req->req_done(req->done_arg, RTEMS_SUCCESSFUL, 0);
	return 0;
}


/* mmcsd_ioctl --
 *     IOCTL handler for RAM disk device.
 *
 * PARAMETERS:
 *      dev  - device number (major, minor number)
 *      req  - IOCTL request code
 *      argp - IOCTL argument
 *
 * RETURNS:
 *     IOCTL return value
 */
static int
mmcsd_ioctl(dev_t dev, uint32_t req, void *argp)
{
	MMCSD_Dev *device=NULL;
	unsigned minor;

	minor=rtems_filesystem_dev_minor_t(dev);
	if(minor<MMCSD_DEVICE_NUM)
		device=g_mmcsd_devices[minor];
	if(!device){
		FAILED(("mmcsd_ioctl: no device for minor %ud\n",minor);
		errno = ENODEV;
		return -1;
	}

	switch (req)
	{
		case BLKIO_REQUEST:
		{
			blkdev_request *r = argp;
			switch (r->req)
			{
				case BLKDEV_REQ_READ:
					return mmcsd_read(device, r);

				case BLKDEV_REQ_WRITE:
					return mmcsd_write(device, r);

				default:
					errno = EBADRQC;
					return -1;
			}
			break;
		}

		default:
			errno = EBADRQC;
			return -1;
	}
}



/* mmcsd_initialize --
 *     RAM disk device driver initialization. Run through RAM disk
 *     configuration information and configure appropriate RAM disks.
 *
 * PARAMETERS:
 *     major - RAM disk major device number
 *     minor - minor device number, not applicable
 *     arg   - initialization argument, not applicable
 *
 * RETURNS:
 *     none
 */
rtems_device_driver
mmcsd_initialize(
    rtems_device_major_number major,
    rtems_device_minor_number minor,
    void *arg)
{
	rtems_status_code rc;
	dev_t dev;
	MMCSD_Dev *device=NULL;
	int ret;

	if(minor<MMCSD_DEVICE_NUM){
		device=g_mmcsd_devices[minor];
	}
	if(!device){
		FAILED(("mmcsd_initialize: no device for minor %ud\n",minor);
		errno = ENODEV;
		return -1;
	}

	rc = rtems_disk_io_initialize();
	if (rc != RTEMS_SUCCESSFUL)
		return rc;

	ret=mmcsd_open_device(device);
        if (ret != MMCSD_OK){
		FAILED(("mmcsd_initialize: mmcsd_open_device failed\n");
		return RTEMS_NOT_DEFINED;
	}

	dev = rtems_filesystem_make_dev_t(major, 0);
	rc = rtems_disk_create_phys(dev, device->blksize, device->sect_num,
				mmcsd_ioctl, MMCSD_DEVICE_NAME);

	return RTEMS_SUCCESSFUL;
}

static rtems_device_major_number mmcsd_major;
static rtems_driver_address_table mmcsd_driver_table={
		.initialization_entry= mmcsd_initialize,	/* initialization procedure */
		.open_entry= rtems_blkdev_generic_open,		/* open request procedure */
		.close_entry= rtems_blkdev_generic_close,	/* close request procedure */
		.read_entry= rtems_blkdev_generic_read,		/* read request procedure */
		.write_entry= rtems_blkdev_generic_write,	/* write request procedure */
		.control_entry= rtems_blkdev_generic_ioctl,	/* special functions procedure */
	};

rtems_status_code
mmcsd_driver_init(void)
{
	int ret;
	rtems_status_code  status;

	ret=mmcsd_init();
        if (ret != MMCSD_OK){
		FAILED(("mmcsd_driver_init: low level initialization (mmcsd_init) failed\n");
		return RTEMS_NOT_DEFINED;
	}

	status = rtems_io_register_driver(0,&mmcsd_driver_table,&mmcsd_major);
        if (status != RTEMS_SUCCESSFUL){
		FAILED(("mmcsd_driver_init: rtems_io_register_driver %s\n",rtems_status_text(status));
		return status;
	}

	status = rtems_io_register_name( MMCSD_DEVICE_NAME, mmcsd_major, 0);
        if (status != RTEMS_SUCCESSFUL){
		FAILED(("mmcsd_driver_init: rtems_io_register_name %s\n",rtems_status_text(status));
		return status;
	}

	return RTEMS_SUCCESSFUL;
}

void mmcsd_hexdump(void *buff, unsigned long addr, int len)
{
	int i;
	uint8_t *p=buff;

	while(len){
		printf("%08lX:",addr);
		i=len>16?16:len;
		addr+=i;
		len-=i;
		while(i--){
			printf("%02X%c",*(p++),i?' ':'\n');
		}
	}
}


int mmcsd_test_read(char *arg)
{
	int ret;
	void *buff;
	int buff_size;
	int sector=0;
	int nr_sector=1;
	MMCSD_Dev *device=g_mmcsd_devices[0];

	if(arg)
		sector=atoi(arg);

	buff_size=nr_sector<<device->blksize_bit;
	buff=malloc(buff_size);
	if(!buff)
		return -1;

	ret=mmcsd_read_device(device, buff, sector, nr_sector);
	if(ret==MMCSD_OK)
		mmcsd_hexdump(buff, sector<<device->blksize_bit, buff_size);
	else
		printf("mmcsd_read_device failed\n");

  	return 0;
}

int mmcsd_test_write(char *arg)
{
	int ret;
	void *buff;
	int buff_size;
	int sector=4;
	int nr_sector=1;
	int i;
	uint8_t *p;
	MMCSD_Dev *device=g_mmcsd_devices[0];

	if(arg)
		sector=atoi(arg);

	buff_size=nr_sector<<device->blksize_bit;
	buff=malloc(buff_size);
	if(!buff)
		return -1;

	p=(uint8_t*)buff;
	for(i=0;i<buff_size;i++){
		*(p++)=i;
	}

	ret=mmcsd_write_device(device, buff, sector, nr_sector);
	if(ret==MMCSD_OK)
		printf("mmcsd_write_device success\n");
	else
		printf("mmcsd_write_device failed\n");

  	return 0;
}

void mmcsd_print_state(void)
{
	printf("MMCSD state:\n");
	printf("IRQ count: %u\n", (unsigned)g_mmcsd_irq_count);
	printf("MMCSD_STR/P_CLK:0x%04x\n",_reg_MMCSD_STR_STP_CLK);
	printf("MMCSD_STATUS:   0x%04x\n",_reg_MMCSD_STATUS);
	printf("MMCSD_CLK_RATE: 0x%04x\n",_reg_MMCSD_CLK_RATE);
	printf("MMCSD_CMD_DAT_C:0x%04x\n",_reg_MMCSD_CMD_DAT_CONT);
	printf("MMCSD_RES_TO:   0x%04x\n",_reg_MMCSD_RES_TO);
	printf("MMCSD_READ_TO:  0x%04x\n",_reg_MMCSD_READ_TO);
	printf("MMCSD_BLK_LEN:  0x%04x\n",_reg_MMCSD_BLK_LEN);
	printf("MMCSD_NOB:      0x%04x\n",_reg_MMCSD_NOB);
	printf("MMCSD_REV_NO:   0x%04x\n",_reg_MMCSD_REV_NO);
	printf("MMCSD_INT_MASK: 0x%04x\n",_reg_MMCSD_INT_MASK);
	printf("MMCSD_CMD:      0x%04x\n",_reg_MMCSD_CMD);
	printf("MMCSD_ARGH:     0x%04x\n",_reg_MMCSD_ARGH);
	printf("MMCSD_ARGL:     0x%04x\n",_reg_MMCSD_ARGL);
/*	printf("MMCSD_RES_FIFO: 0x%04x\n",_reg_MMCSD_RES_FIFO);*/
/*	printf("MMCSD_BUFFER_A: 0x%04x\n",_reg_MMCSD_BUFFER_ACCESS);*/
}

static unsigned aitc_get_field(volatile uint32_t *addr, unsigned fld_bits, unsigned fld_num)
{
	unsigned bit_offs=fld_num*fld_bits;

	addr+=(64*fld_bits-bit_offs-1)/32;
	bit_offs=bit_offs%32;
	return ((*addr)>>bit_offs)&((1<<fld_bits)-1);
}

void mmcsd_test_irqinfo(int irq_number)
{
	uint32_t ctl;

	ctl = _reg_AITC_INTCNTL;
	printf("AITC state: NIAD=%i FIAD=%i NIMASK=0x%x\n",
		ctl&MX1_AITC_INTCNTL_NIAD_m?1:0,
		ctl&MX1_AITC_INTCNTL_FIAD_m?1:0,
		_reg_AITC_NIMASK);

	printf("INTENABLE = 0x%08x:0x%08x\n",
		_reg_AITC_INTENABLEH,
		_reg_AITC_INTENABLEL);

	printf("INTTYPE   = 0x%08x:0x%08x\n",
		_reg_AITC_INTTYPEH,
		_reg_AITC_INTTYPEL);

	printf("INTTYPE   = 0x%08x:0x%08x\n",
		_reg_AITC_INTTYPEH,
		_reg_AITC_INTTYPEL);

	printf("INTSRC    = 0x%08x:0x%08x\n",
		_reg_AITC_INTSRCH,
		_reg_AITC_INTSRCL);

	printf("irq %d %c %c prio %2d %s\n",
		irq_number,
		aitc_get_field(&_reg_AITC_INTENABLEH, 1, irq_number)?'E':'D',
		aitc_get_field(&_reg_AITC_INTTYPEH, 1, irq_number)?'F':'N',
		aitc_get_field(&_reg_AITC_NIPRIORITY(7), 4, irq_number),
		aitc_get_field(&_reg_AITC_INTSRCH, 1, irq_number)?"pend":"----");
}


fstab_t fs_table[] = {
  {
    "/dev/mmcsd1","/mnt/mmcsd",
    &msdos_ops, 1?RTEMS_FILESYSTEM_READ_ONLY:RTEMS_FILESYSTEM_READ_WRITE,
    FSMOUNT_MNT_OK | FSMOUNT_MNTPNT_CRTERR | FSMOUNT_MNT_FAILED,
    0
  }
};

int mmcsd_partitions_number;

int mmcsd_command_forshell(int argc, char **argv)
{
	int ret;
	int i;

	if(argc>=2){
		if(!strcmp(argv[1],"irqinfo")){
			i = MMCSD_IRQ;
			if(argc>2)
				i = atoi(argv[2]);
			mmcsd_test_irqinfo(i);
			return 0;
		}
		if(!strcmp(argv[1],"p")){
			mmcsd_print_state();
			return 0;
		}
	}

	if(!g_mmcsd_devices[0])
		mmcsd_driver_init();

	if(argc<2)
		return 0;

	if(!strcmp(argv[1],"open")){
		ret=mmcsd_open_device(g_mmcsd_devices[0]);
		printf("mmcsd_open_device returned %d\n",ret);
		return 0;
	}
	if(!strcmp(argv[1],"getmbr")){
		ret=mmcsd_getMBR(g_mmcsd_devices[0]);
		printf("mmcsd_getMBR returned %d\n",ret);
		return 0;
	}
	if(!strcmp(argv[1],"idle")){
		ret=_MMCSD_GoIdleState(g_mmcsd_devices[0]);
		printf("_MMCSD_GoIdleState returned %d\n",ret);
		return 0;
	}
	if(!strcmp(argv[1],"sendopcond")){
		ret=_MMCSD_SendOpCond(g_mmcsd_devices[0], MMCSD_MMC_CARD_VOLTAGE);
		printf("_MMCSD_SendOpCond returned %d\n",ret);
		return 0;
	}
	if(!strcmp(argv[1],"read")){
		mmcsd_test_read(argc>2?argv[2]:NULL);
		return 0;
	}
	if(!strcmp(argv[1],"write")){
		mmcsd_test_write(argc>2?argv[2]:NULL);
		return 0;
	}
	if(!strcmp(argv[1],"mount")){
		if(!mmcsd_partitions_number){
			mmcsd_partitions_number=rtems_ide_part_table_initialize(MMCSD_DEVICE_NAME);
			printf("ide_part_table_initialize returned %d\n",ret);
		}
		ret = rtems_fsmount( fs_table,
                      sizeof(fs_table)/sizeof(fs_table[0]),
                      NULL);
		printf("rtems_fsmount %d\n",ret);
		return 0;
	}
	if(!strcmp(argv[1],"format")){
		if(!mmcsd_partitions_number){
			mmcsd_partitions_number=rtems_ide_part_table_initialize(MMCSD_DEVICE_NAME);
			printf("ide_part_table_initialize returned %d\n",ret);
		}
		printf("Starting formating of the MMC/SD device\n");
		msdos_format("/dev/mmcsd1", NULL);
		return 0;
	}
	return 0;
}
